package com.gic.haoban.manage.service.service.out.impl.hm;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.manage.api.dto.hm.HmLinkDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkStoreDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkStoreSettingDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkWxaDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmLinkSearchQDTO;
import com.gic.haoban.manage.api.service.hm.HmLinkApiService;
import com.gic.haoban.manage.service.entity.hm.TabHmLink;
import com.gic.haoban.manage.service.errorcode.HaoBanErrCode;
import com.gic.haoban.manage.service.service.KeyDataService;
import com.gic.haoban.manage.service.service.hm.HmLinkService;
import com.gic.haoban.manage.service.service.hm.HmLinkStoreService;
import com.gic.member.api.dto.es.MemberDataDTO;
import com.gic.member.api.dto.es.MemberStoreClerkDataDTO;
import com.gic.member.api.service.MemberService;
import com.gic.search.engine.api.dto.ESResponseQuerySingle;
import com.gic.search.engine.api.dto.dynamic.DynamicSearchDTO;
import com.gic.search.engine.api.service.dynamic.ESDataDynamicOperationApiService;

@Service("hmLinkApiService")
public class HmLinkApiServiceImpl implements HmLinkApiService {

    private static final Logger log = LoggerFactory.getLogger(HmLinkApiService.class);

    @Autowired
    private HmLinkService hmLinkService;
    @Autowired
    private HmLinkStoreService hmLinkStoreService;
    @Autowired
    private KeyDataService keyDataService;
    @Autowired
    private MemberService memberService;
    @Autowired
    private ESDataDynamicOperationApiService esApiService;
    @Autowired
    private StoreService storeService;

    @Override
    public ServiceResponse<HmLinkDTO> getByLinkId(String wxEnterpriseId, String enterpriseId, Long linkId) {
        TabHmLink link = this.hmLinkService.getByLinkId(wxEnterpriseId, enterpriseId, linkId);
        if (null != link) {
            HmLinkDTO dto = EntityUtil.changeEntityByJSON(HmLinkDTO.class, link);
            List<HmLinkStoreDTO> storeList = this.hmLinkStoreService.listByLinkId(dto.getWxEnterpriseId(),
                    dto.getEnterpriseId(), linkId);
            dto.setStoreList(storeList);
            return ServiceResponse.success(dto);
        }
        return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "链接不存在");
    }

    @Override
    public ServiceResponse<List<HmLinkDTO>> listForSms(String wxEnterpriseId, String enterprieId, String linkName) {
        return ServiceResponse.success(this.hmLinkService.listForSms(wxEnterpriseId, enterprieId, linkName));
    }

    @Override
    public ServiceResponse<HmLinkDTO> save(HmLinkDTO dto) {
        return ServiceResponse.success(this.hmLinkService.saveLink(dto, dto.getStoreList()));
    }

    @Override
    public ServiceResponse<Page<HmLinkDTO>> listPage(HmLinkSearchQDTO qdto, BasePageInfo basePageInfo) {
        return ServiceResponse.success(this.hmLinkService.listPage(qdto, basePageInfo));
    }

    @Override
    public ServiceResponse<HmLinkWxaDTO> getLinkHmFromWxa(String enterpriseId, String storeId, String clerkId,
            String linkId, String mixPhone, String unionid) {
        log.info("小程序获取活码={},linkId={},mixPhone={}", enterpriseId, linkId, mixPhone);
        Long hmLinkId = Long.parseLong(linkId);
        TabHmLink link = this.hmLinkService.getByLinkId(null, null, hmLinkId);

        // 判断导购是否有活码,如果没有返回错误
        if (StringUtils.isNotBlank(clerkId)) {

        }
        // 直接返回门店的导购活码,如果没有返回错误
        if (StringUtils.isNotBlank(storeId)) {

        }

        HmLinkStoreSettingDTO fwStore = null;
        HmLinkStoreSettingDTO xgStore = null;
        if (StringUtils.isNotBlank(link.getStoreRuleJson())) {
            String storeRuleJson = link.getStoreRuleJson();
            JSONObject storeRuleObj = JSONObject.parseObject(storeRuleJson);
            fwStore = storeRuleObj.getObject("fw_store", HmLinkStoreSettingDTO.class);
            if (null == fwStore || fwStore.getOpen() == 0) {
                fwStore = null;
            }
            xgStore = storeRuleObj.getObject("xg_store", HmLinkStoreSettingDTO.class);
            if (null == xgStore || xgStore.getOpen() == 0) {
                xgStore = null;
            }
        }
        // 会员门店
        if (StringUtils.isNotEmpty(mixPhone) && (null != fwStore || null != xgStore)) {
            String mobile = this.keyDataService.getMobile(mixPhone, linkId);
            log.info("小程序获取活码mobile={}", mobile);
            String memberId = this.memberService.getMemberIdByPhoneNumber(mobile, enterpriseId);
            log.info("小程序获取活码memberId={}", memberId);
            MemberDataDTO member = this.getMemberClerk(enterpriseId, memberId);
            String mainClerkId = member.getMainClerkId();
            List<MemberStoreClerkDataDTO> list = member.getStoreClerkRel() ;
            for(int i=0;i<list.size();i++) {
                
            }
            
        }

        if (StringUtils.isNotBlank(link.getCustomRuleJson())) {
            String customRuleJson = link.getCustomRuleJson();
            JSONObject storeRuleObj = JSONObject.parseObject(customRuleJson);
            int openFlag = storeRuleObj.getIntValue("open");
            if (openFlag == 1) {
                HmLinkStoreSettingDTO cityStore = storeRuleObj.getObject("city_store", HmLinkStoreSettingDTO.class);
            }
        }

        // 查询活码
        Long hmId = null;
        String key = this.keyDataService.saveHmData(unionid, hmLinkId, hmId);
        // 生成动态活码
        
        return null;
    }

    // 冗余会员信息
    private MemberDataDTO getMemberClerk(String enterpriseId, String memberId) {
        String returnFields = "memberId,mainClerkId,storeClerkRel";
        JSONObject memberObj = this.queryDataSingle(enterpriseId, memberId, returnFields);
        if (null == memberObj) {
            return null;
        }
        log.info("会员信息={}", memberObj.toJSONString());
        MemberDataDTO memberDataDTO = memberObj.toJavaObject(MemberDataDTO.class);
        return memberDataDTO;
    }

    public JSONObject queryDataSingle(String enterpriseId, String memberId, String returnFields) {
        JSONObject searchRes = null;
        try {
            JSONObject searchJson = new JSONObject();
            DynamicSearchDTO search = new DynamicSearchDTO();
            searchJson.put("id", memberId);
            search.setSearchJson(searchJson);
            search.setEnterpriseId(enterpriseId);
            search.setColumnCategoryCode("member");
            if (StringUtils.isNotBlank(returnFields)) {
                search.setReturnFileds(returnFields);
            }
            ESResponseQuerySingle single = esApiService.queryDataSingle(search);
            searchRes = single.getRes();
        } catch (Exception e) {
            log.warn("从搜索引擎搜索会员数据异常memberId: {}", memberId, e);
        }
        return searchRes;
    }

}
